@@ -1,6 +1,6 @@
 /* Support for the generic parts of PE/PEI; the common executable parts.
    Copyright 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005 Free Software Foundation, Inc.
+   2005, 2006 Free Software Foundation, Inc.
    Written by Cygnus Solutions.
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -843,18 +843,17 @@ _bfd_XXi_swap_scnhdr_out (bfd * abfd, void * in, void * out)
   SCNHDR *scnhdr_ext = (SCNHDR *) out;
   unsigned int ret = SCNHSZ;
   bfd_vma ps;
-  bfd_vma ss;
+    bfd_vma ss;
 
-  memcpy (scnhdr_ext->s_name, scnhdr_int->s_name, sizeof (scnhdr_int->s_name));
+    memcpy (scnhdr_ext->s_name, scnhdr_int->s_name, sizeof (scnhdr_int->s_name));
 
-  PUT_SCNHDR_VADDR (abfd,
+    PUT_SCNHDR_VADDR (abfd,
 		    ((scnhdr_int->s_vaddr
 		      - pe_data (abfd)->pe_opthdr.ImageBase)
 		     & 0xffffffff),
-		    scnhdr_ext->s_vaddr);
-
-  /* NT wants the size data to be rounded up to the next
-     NT_FILE_ALIGNMENT, but zero if it has no content (as in .bss,
+  		    scnhdr_ext->s_vaddr);
+  
+    /* NT wants the size data to be rounded up to the next     NT_FILE_ALIGNMENT, but zero if it has no content (as in .bss,
      sometimes).  */
   if ((scnhdr_int->s_flags & IMAGE_SCN_CNT_UNINITIALIZED_DATA) != 0)
     {
@@ -1668,7 +1667,7 @@ static const char * const tbl[] =
   "RESERVED1",
   "MIPS_JMPADDR16",
   "DIR64",
-  "HIGH3ADJ"
+  "HIGH3ADJ",
   "UNKNOWN",   /* MUST be last.  */
 };
 
